from pyspark.sql import Column
import pyspark.sql.functions as fn
from pyspark.sql.types import (
    DoubleType,
    LongType,
)

from .udfs import is_scientific
import trend_etl.schema.schema as schema


def normalize_reddit(data_after_parsed: Column) -> Column:
    return data_after_parsed.withField(
        "created_utc",
        fn.when(
            is_scientific(data_after_parsed.getField("created_utc")),
            data_after_parsed.getField("created_utc")
            .cast(DoubleType())
            .cast(LongType()),
        ).otherwise(data_after_parsed.getField("created_utc").cast(LongType())),
    )


def parse_facebook_groups(json_field: Column) -> Column:
    return fn.from_json(json_field, schema.facebook_groups_schema, {"mode": "FAILFAST"})


def parse_facebook_posts(json_field: Column) -> Column:
    return fn.from_json(json_field, schema.facebook_posts_schema, {"mode": "FAILFAST"})


def parse_facebook_comments(json_field: Column) -> Column:
    return fn.from_json(
        json_field, schema.facebook_comments_schema, {"mode": "FAILFAST"}
    )


def parse_subreddits(json_field: Column) -> Column:
    return fn.from_json(json_field, schema.subreddits_schema, {"mode": "FAILFAST"})


def parse_pushshift_submissions(json_field: Column) -> Column:
    return fn.from_json(
        json_field, schema.pushshift_submissions_schema, {"mode": "FAILFAST"}
    )


def parse_reddit_submissions(json_field: Column) -> Column:
    return fn.from_json(json_field, schema.submissions_schema, {"mode": "FAILFAST"})


def parse_pushshift_replies(json_field: Column) -> Column:
    return fn.from_json(
        json_field, schema.pushshift_replies_schema, {"mode": "FAILFAST"}
    )


def parse_reddit_replies(json_field: Column) -> Column:
    return fn.from_json(json_field, schema.replies_schema, {"mode": "FAILFAST"})


def parse_guilds(json_field: Column) -> Column:
    return fn.from_json(json_field, schema.discord_guild_schema, {"mode": "FAILFAST"})


def parse_channels(json_field: Column) -> Column:
    return fn.from_json(json_field, schema.discord_channel_schema, {"mode": "FAILFAST"})


def parse_messages(json_field: Column) -> Column:
    return fn.from_json(json_field, schema.discord_message_schema, {"mode": "FAILFAST"})


def parse_reactions(json_field: Column) -> Column:
    return fn.from_json(
        json_field, schema.discord_reaction_schema, {"mode": "FAILFAST"}
    )


def parse_members(json_field: Column) -> Column:
    return fn.from_json(json_field, schema.discord_member_schema, {"mode": "FAILFAST"})
