import trend_etl.functions.ingest_versioning as ingest
import trend_etl.functions.udfs as udfs
import trend_etl.schema.types as types

## SUBREDDITS
subreddit_canonical: types.Canonical = [
    ("accept_followers", ingest.boolean_to_binary),
    ("accounts_active", ingest.direct_to_binary),
    ("accounts_active_is_fuzzed", ingest.boolean_to_binary),
    ("active_user_count", ingest.direct_to_binary),
    ("advertiser_category", ingest.direct_to_binary),
    ("all_original_content", ingest.boolean_to_binary),
    ("allow_chat_post_creation", ingest.boolean_to_binary),
    ("allow_discovery", ingest.boolean_to_binary),
    ("allow_galleries", ingest.boolean_to_binary),
    ("allow_images", ingest.boolean_to_binary),
    ("allow_polls", ingest.boolean_to_binary),
    ("allow_prediction_contributors", ingest.boolean_to_binary),
    ("allow_predictions", ingest.boolean_to_binary),
    ("allow_predictions_tournament", ingest.boolean_to_binary),
    ("allow_talks", ingest.boolean_to_binary),
    ("allow_videogifs", ingest.boolean_to_binary),
    ("allow_videos", ingest.boolean_to_binary),
    ("allowed_media_in_comments", ingest.array_direct_to_binary),
    ("banner_background_color", ingest.direct_to_binary),
    ("banner_background_image", ingest.direct_to_binary),
    ("banner_img", ingest.direct_to_binary),
    ("banner_size", ingest.array_direct_to_binary),
    ("can_assign_link_flair", ingest.boolean_to_binary),
    ("can_assign_user_flair", ingest.boolean_to_binary),
    ("collapse_deleted_comments", ingest.boolean_to_binary),
    (
        "comment_contribution_settings",
        ingest.reddit_comment_contribution_settings_to_binary,
    ),
    ("comment_score_hide_mins", ingest.direct_to_binary),
    ("community_icon", ingest.direct_to_binary),
    ("community_reviewed", ingest.boolean_to_binary),
    ("description", ingest.direct_to_binary),
    ("description_html", ingest.direct_to_binary),
    ("disable_contributor_requests", ingest.boolean_to_binary),
    ("display_name", ingest.direct_to_binary),
    ("display_name_prefixed", ingest.direct_to_binary),
    ("emojis_custom_size", ingest.array_direct_to_binary),
    ("emojis_enabled", ingest.boolean_to_binary),
    ("free_form_reports", ingest.boolean_to_binary),
    ("has_menu_widget", ingest.boolean_to_binary),
    ("header_img", ingest.direct_to_binary),
    ("header_size", ingest.array_direct_to_binary),
    ("header_title", ingest.direct_to_binary),
    ("hide_ads", ingest.boolean_to_binary),
    ("icon_img", ingest.direct_to_binary),
    ("icon_size", ingest.array_direct_to_binary),
    ("id", ingest.direct_to_binary),
    ("interstitial_warning_message", ingest.direct_to_binary),
    ("is_chat_post_feature_enabled", ingest.boolean_to_binary),
    ("is_crosspostable_subreddit", ingest.boolean_to_binary),
    ("is_enrolled_in_new_modmail", ingest.direct_to_binary),
    ("key_color", ingest.direct_to_binary),
    ("lang", ingest.direct_to_binary),
    ("link_flair_enabled", ingest.boolean_to_binary),
    ("link_flair_position", ingest.direct_to_binary),
    ("mobile_banner_image", ingest.direct_to_binary),
    ("name", ingest.direct_to_binary),
    ("notification_level", ingest.direct_to_binary),
    ("original_content_tag_enabled", ingest.boolean_to_binary),
    ("over18", ingest.boolean_to_binary),
    ("prediction_leaderboard_entry_type", ingest.direct_to_binary),
    ("primary_color", ingest.direct_to_binary),
    ("public_description", ingest.direct_to_binary),
    ("public_description_html", ingest.direct_to_binary),
    ("public_traffic", ingest.boolean_to_binary),
    ("quarantine", ingest.boolean_to_binary),
    ("restrict_commenting", ingest.boolean_to_binary),
    ("restrict_posting", ingest.boolean_to_binary),
    ("should_archive_posts", ingest.boolean_to_binary),
    ("should_show_media_in_comments_setting", ingest.boolean_to_binary),
    ("show_media", ingest.boolean_to_binary),
    ("show_media_preview", ingest.boolean_to_binary),
    ("spoilers_enabled", ingest.boolean_to_binary),
    ("submission_type", ingest.direct_to_binary),
    ("submit_link_label", ingest.direct_to_binary),
    ("submit_text", ingest.direct_to_binary),
    ("submit_text_html", ingest.direct_to_binary),
    ("submit_text_label", ingest.direct_to_binary),
    ("subreddit_type", ingest.direct_to_binary),
    ("subscribers", ingest.direct_to_binary),
    ("suggested_comment_sort", ingest.direct_to_binary),
    ("title", ingest.direct_to_binary),
    ("url", ingest.direct_to_binary),
    ("user_can_flair_in_sr", ingest.boolean_to_binary),
    ("user_flair_background_color", ingest.direct_to_binary),
    ("user_flair_css_class", ingest.direct_to_binary),
    ("user_flair_enabled_in_sr", ingest.boolean_to_binary),
    ("user_flair_position", ingest.direct_to_binary),
    ("user_flair_richtext", ingest.array_direct_to_binary),
    ("user_flair_template_id", ingest.direct_to_binary),
    ("user_flair_text", ingest.direct_to_binary),
    ("user_flair_text_color", ingest.direct_to_binary),
    ("user_flair_type", ingest.direct_to_binary),
    ("user_has_favorited", ingest.boolean_to_binary),
    ("user_is_banned", ingest.boolean_to_binary),
    ("user_is_contributor", ingest.boolean_to_binary),
    ("user_is_moderator", ingest.boolean_to_binary),
    ("user_is_muted", ingest.boolean_to_binary),
    ("user_is_subscriber", ingest.boolean_to_binary),
    ("user_sr_flair_enabled", ingest.boolean_to_binary),
    ("user_sr_theme_enabled", ingest.boolean_to_binary),
    ("videostream_links_count", ingest.direct_to_binary),
    ("whitelist_status", ingest.direct_to_binary),
    ("wiki_enabled", ingest.boolean_to_binary),
    ("wls", ingest.direct_to_binary),
    ("created_utc", ingest.direct_to_binary),
]

## SUBMISSIONS

reddit_submission_canonical: types.Canonical = [
    ("subreddit_id", ingest.direct_to_binary),
    ("name", ingest.direct_to_binary),
    ("approved_at_utc", ingest.direct_to_binary),
    ("selftext", ingest.direct_to_binary),
    ("author_fullname", ingest.direct_to_binary),
    ("saved", ingest.boolean_to_binary),
    ("mod_reason_title", ingest.direct_to_binary),
    ("gilded", ingest.direct_to_binary),
    ("clicked", ingest.boolean_to_binary),
    ("title", ingest.direct_to_binary),
    ("link_flair_richtext", ingest.reddit_link_flair_richtext_to_binary),
    ("hidden", ingest.boolean_to_binary),
    ("pwls", ingest.direct_to_binary),
    ("link_flair_css_class", ingest.direct_to_binary),
    ("downs", ingest.direct_to_binary),
    ("thumbnail_height", ingest.direct_to_binary),
    ("top_awarded_type", ingest.direct_to_binary),
    ("hide_score", ingest.boolean_to_binary),
    ("quarantine", ingest.boolean_to_binary),
    ("link_flair_text_color", ingest.direct_to_binary),
    ("upvote_ratio", udfs.double_to_binary),
    ("author_flair_background_color", ingest.direct_to_binary),
    ("subreddit_type", ingest.direct_to_binary),
    ("ups", ingest.direct_to_binary),
    ("total_awards_received", ingest.direct_to_binary),
    ("thumbnail_width", ingest.direct_to_binary),
    ("author_flair_template_id", ingest.direct_to_binary),
    ("is_original_content", ingest.boolean_to_binary),
    ("user_reports", ingest.array_direct_to_binary),
    ("is_reddit_media_domain", ingest.boolean_to_binary),
    ("is_meta", ingest.boolean_to_binary),
    ("category", ingest.direct_to_binary),
    ("link_flair_text", ingest.direct_to_binary),
    ("can_mod_post", ingest.boolean_to_binary),
    ("score", ingest.direct_to_binary),
    ("approved_by", ingest.direct_to_binary),
    ("is_created_from_ads_ui", ingest.boolean_to_binary),
    ("author_premium", ingest.boolean_to_binary),
    ("thumbnail", ingest.direct_to_binary),
    ("edited", ingest.direct_to_binary),
    ("author_flair_css_class", ingest.direct_to_binary),
    ("author_flair_richtext", ingest.reddit_author_flair_richtext_to_binary),
    ("gildings", ingest.reddit_gildings_to_binary),
    ("post_hint", ingest.direct_to_binary),
    ("content_categories", ingest.direct_to_binary),
    ("is_self", ingest.boolean_to_binary),
    ("mod_note", ingest.direct_to_binary),
    ("link_flair_type", ingest.direct_to_binary),
    ("wls", ingest.direct_to_binary),
    ("removed_by_category", ingest.direct_to_binary),
    ("banned_by", ingest.direct_to_binary),
    ("author_flair_type", ingest.direct_to_binary),
    ("domain", ingest.direct_to_binary),
    ("allow_live_comments", ingest.boolean_to_binary),
    ("likes", ingest.direct_to_binary),
    ("suggested_sort", ingest.direct_to_binary),
    ("banned_at_utc", ingest.direct_to_binary),
    ("view_count", ingest.direct_to_binary),
    ("archived", ingest.boolean_to_binary),
    ("no_follow", ingest.boolean_to_binary),
    ("is_crosspostable", ingest.boolean_to_binary),
    ("pinned", ingest.boolean_to_binary),
    ("over_18", ingest.boolean_to_binary),
    ("all_awardings", ingest.array_direct_to_binary),
    ("awarders", ingest.array_direct_to_binary),
    ("media_only", ingest.boolean_to_binary),
    ("can_gild", ingest.boolean_to_binary),
    ("spoiler", ingest.boolean_to_binary),
    ("locked", ingest.boolean_to_binary),
    ("author_flair_text", ingest.direct_to_binary),
    ("treatment_tags", ingest.array_direct_to_binary),
    ("visited", ingest.boolean_to_binary),
    ("removed_by", ingest.direct_to_binary),
    ("num_reports", ingest.direct_to_binary),
    ("distinguished", ingest.direct_to_binary),
    ("author_is_blocked", ingest.boolean_to_binary),
    ("mod_reason_by", ingest.direct_to_binary),
    ("removal_reason", ingest.direct_to_binary),
    ("link_flair_background_color", ingest.direct_to_binary),
    ("is_robot_indexable", ingest.boolean_to_binary),
    ("report_reasons", ingest.direct_to_binary),
    ("author", ingest.direct_to_binary),
    ("discussion_type", ingest.direct_to_binary),
    ("num_comments", ingest.direct_to_binary),
    ("send_replies", ingest.boolean_to_binary),
    ("whitelist_status", ingest.direct_to_binary),
    ("contest_mode", ingest.boolean_to_binary),
    ("mod_reports", ingest.array_direct_to_binary),
    ("author_patreon_flair", ingest.boolean_to_binary),
    ("author_flair_text_color", ingest.direct_to_binary),
    ("permalink", ingest.direct_to_binary),
    ("parent_whitelist_status", ingest.direct_to_binary),
    ("stickied", ingest.boolean_to_binary),
    ("url", ingest.direct_to_binary),
    ("subreddit_subscribers", ingest.direct_to_binary),
    ("num_crossposts", ingest.direct_to_binary),
    ("is_video", ingest.boolean_to_binary),
    ("created_utc", ingest.direct_to_binary),
]

reddit_submission_text_canonical: types.Canonical = [
    ("selftext", ingest.direct_to_binary),
]

## REPLIES

reddit_reply_canonical: types.Canonical = [
    ("all_awardings", ingest.reddit_all_awardings_to_binary),
    ("approved_at_utc", ingest.direct_to_binary),
    ("approved_by", ingest.direct_to_binary),
    ("archived", ingest.boolean_to_binary),
    ("associated_award", ingest.direct_to_binary),
    ("author", ingest.direct_to_binary),
    ("author_cakeday", ingest.boolean_to_binary),
    ("author_flair_background_color", ingest.direct_to_binary),
    ("author_flair_css_class", ingest.direct_to_binary),
    ("author_flair_richtext", ingest.reddit_author_flair_richtext_to_binary),
    ("author_flair_template_id", ingest.direct_to_binary),
    ("author_flair_text", ingest.direct_to_binary),
    ("author_flair_text_color", ingest.direct_to_binary),
    ("author_flair_type", ingest.direct_to_binary),
    ("author_fullname", ingest.direct_to_binary),
    ("author_is_blocked", ingest.boolean_to_binary),
    ("author_patreon_flair", ingest.boolean_to_binary),
    ("author_premium", ingest.boolean_to_binary),
    ("awarders", ingest.array_direct_to_binary),
    ("banned_at_utc", ingest.direct_to_binary),
    ("banned_by", ingest.direct_to_binary),
    ("body", ingest.direct_to_binary),
    ("can_gild", ingest.boolean_to_binary),
    ("can_mod_post", ingest.boolean_to_binary),
    ("collapsed", ingest.boolean_to_binary),
    ("collapsed_because_crowd_control", ingest.direct_to_binary),
    ("collapsed_reason", ingest.direct_to_binary),
    ("collapsed_reason_code", ingest.direct_to_binary),
    ("comment_type", ingest.direct_to_binary),
    ("controversiality", ingest.direct_to_binary),
    ("distinguished", ingest.direct_to_binary),
    ("downs", ingest.direct_to_binary),
    ("editable", ingest.boolean_to_binary),
    ("edited", ingest.direct_to_binary),
    ("gilded", ingest.direct_to_binary),
    ("gildings", ingest.reddit_gildings_to_binary),
    ("is_submitter", ingest.boolean_to_binary),
    ("likes", ingest.direct_to_binary),
    ("link_author", ingest.direct_to_binary),
    ("link_id", ingest.direct_to_binary),
    ("link_permalink", ingest.direct_to_binary),
    ("link_title", ingest.direct_to_binary),
    ("link_url", ingest.direct_to_binary),
    ("locked", ingest.boolean_to_binary),
    ("mod_note", ingest.direct_to_binary),
    ("mod_reason_by", ingest.direct_to_binary),
    ("mod_reason_title", ingest.direct_to_binary),
    ("mod_reports", ingest.array_direct_to_binary),
    ("name", ingest.direct_to_binary),
    ("no_follow", ingest.boolean_to_binary),
    ("num_comments", ingest.direct_to_binary),
    ("num_reports", ingest.direct_to_binary),
    ("over_18", ingest.boolean_to_binary),
    ("parent_id", ingest.direct_to_binary),
    ("permalink", ingest.direct_to_binary),
    ("quarantine", ingest.boolean_to_binary),
    ("removal_reason", ingest.direct_to_binary),
    ("replies", ingest.direct_to_binary),
    ("report_reasons", ingest.direct_to_binary),
    ("saved", ingest.boolean_to_binary),
    ("score", ingest.direct_to_binary),
    ("score_hidden", ingest.boolean_to_binary),
    ("send_replies", ingest.boolean_to_binary),
    ("stickied", ingest.boolean_to_binary),
    ("subreddit_id", ingest.direct_to_binary),
    ("subreddit_type", ingest.direct_to_binary),
    ("top_awarded_type", ingest.direct_to_binary),
    ("total_awards_received", ingest.direct_to_binary),
    ("treatment_tags", ingest.array_direct_to_binary),
    ("unrepliable_reason", ingest.direct_to_binary),
    ("ups", ingest.direct_to_binary),
    ("user_reports", ingest.array_direct_to_binary),
    ("created_utc", ingest.direct_to_binary),
]

reddit_reply_text_canonical: types.Canonical = [
    ("body", ingest.direct_to_binary),
]

## GROUPS

facebook_group_canonical: types.Canonical = [
    ("id", ingest.direct_to_binary),
    ("email", ingest.direct_to_binary),
    ("description", ingest.direct_to_binary),
    ("created_time", ingest.direct_to_binary),
    ("name", ingest.direct_to_binary),
    ("privacy", ingest.direct_to_binary),
    ("member_count", ingest.direct_to_binary),
]

## POSTS

facebook_post_canonical: types.Canonical = [
    ("id", ingest.direct_to_binary),
    ("group_id", ingest.direct_to_binary),
    ("group_name", ingest.direct_to_binary),
    ("created_time", ingest.direct_to_binary),
    ("updated_time", ingest.direct_to_binary),
    ("link", ingest.direct_to_binary),
    ("story", ingest.direct_to_binary),
    # NOTE: Ingore this field from bronze (use anonymized version
    # ("message", ingest.direct_to_binary),
    ("anonymized_message", ingest.direct_to_binary),
    # NOTE: Ingore this field from bronze (use anonymized version)
    # ("name", ingest.direct_to_binary),
    ("anonymized_name", ingest.direct_to_binary),
    ("permalink_url", ingest.direct_to_binary),
    ("type", ingest.direct_to_binary),
    ("reactions_angry", ingest.facebook_reaction_to_binary),
    ("reactions_care", ingest.facebook_reaction_to_binary),
    ("reactions_haha", ingest.facebook_reaction_to_binary),
    ("reactions_like", ingest.facebook_reaction_to_binary),
    ("reactions_love", ingest.facebook_reaction_to_binary),
    ("reactions_none", ingest.facebook_reaction_to_binary),
    ("reactions_pride", ingest.facebook_reaction_to_binary),
    ("reactions_sad", ingest.facebook_reaction_to_binary),
    ("reactions_thankful", ingest.facebook_reaction_to_binary),
    ("reactions_wow", ingest.facebook_reaction_to_binary),
]

facebook_post_text_canonical: types.Canonical = [
    ("anonymized_message", ingest.direct_to_binary),
]

## COMMENTS

facebook_comment_canonical: types.Canonical = [
    ("id", ingest.direct_to_binary),
    ("group_id", ingest.direct_to_binary),
    ("group_name", ingest.direct_to_binary),
    ("created_time", ingest.direct_to_binary),
    # NOTE: Ingore this field from bronze (use anonymized version)
    # ("message", ingest.direct_to_binary),
    ("anonymized_message", ingest.direct_to_binary),
    ("parent", ingest.facebook_parent_to_binary),
    ("permalink_url", ingest.direct_to_binary),
    ("post_id", ingest.direct_to_binary),
    ("reactions_angry", ingest.facebook_reaction_to_binary),
    ("reactions_care", ingest.facebook_reaction_to_binary),
    ("reactions_haha", ingest.facebook_reaction_to_binary),
    ("reactions_like", ingest.facebook_reaction_to_binary),
    ("reactions_love", ingest.facebook_reaction_to_binary),
    ("reactions_none", ingest.facebook_reaction_to_binary),
    ("reactions_pride", ingest.facebook_reaction_to_binary),
    ("reactions_sad", ingest.facebook_reaction_to_binary),
    ("reactions_thankful", ingest.facebook_reaction_to_binary),
    ("reactions_wow", ingest.facebook_reaction_to_binary),
]

facebook_comment_text_canonical: types.Canonical = [
    ("anonymized_message", ingest.direct_to_binary),
]

## GUILDS

discord_guild_canonical: types.Canonical = [
    ("id", ingest.direct_to_binary),
    ("name", ingest.direct_to_binary),
    ("icon", ingest.direct_to_binary),
    ("owner", ingest.boolean_to_binary),
    ("owner_id", ingest.direct_to_binary),
    ("permissions", ingest.direct_to_binary),
    ("features", ingest.array_direct_to_binary),
    ("verification_level", ingest.direct_to_binary),
    ("approximate_member_count", ingest.direct_to_binary),
    ("approximate_presence_count", ingest.direct_to_binary),
    ("description", ingest.direct_to_binary),
]

## CHANNELS

discord_channel_canonical: types.Canonical = [
    ("id", ingest.direct_to_binary),
    ("type", ingest.direct_to_binary),
    ("guild_id", ingest.direct_to_binary),
    ("position", ingest.direct_to_binary),
    (
        "permission_overwrites",
        ingest.discord_permission_overwrites_to_binary,
    ),
    ("name", ingest.direct_to_binary),
    ("topic", ingest.direct_to_binary),
    ("nsfw", ingest.boolean_to_binary),
    ("owner_id", ingest.direct_to_binary),
    ("rate_limit_per_user", ingest.direct_to_binary),
    ("icon", ingest.direct_to_binary),
    ("parent_id", ingest.direct_to_binary),
    ("message_count", ingest.direct_to_binary),
    ("member_count", ingest.direct_to_binary),
    (
        "thread_metadata",
        ingest.discord_thread_metadata_to_binary,
    ),
    ("permissions", ingest.direct_to_binary),
    ("flags", ingest.direct_to_binary),
    ("total_message_sent", ingest.direct_to_binary),
    (
        "available_tags",
        ingest.discord_available_tags_to_binary,
    ),
    ("applied_tags", ingest.array_direct_to_binary),
    (
        "default_reaction_emoji",
        ingest.discord_default_reaction_emoji_to_binary,
    ),
    ("default_forum_layout", ingest.direct_to_binary),
]


## MESSAGES

discord_message_canonical: types.Canonical = [
    ("id", ingest.direct_to_binary),
    ("guild_id", ingest.direct_to_binary),
    ("channel_id", ingest.direct_to_binary),
    ("content", ingest.direct_to_binary),
    ("timestamp", ingest.direct_to_binary),
    ("edited_timestamp", ingest.direct_to_binary),
    ("tts", ingest.boolean_to_binary),
    ("mention_everyone", ingest.boolean_to_binary),
    ("embeds", ingest.discord_embeds_to_binary),
    ("pinned", ingest.boolean_to_binary),
    ("type", ingest.direct_to_binary),
    ("message_reference", ingest.discord_message_reference_to_binary),
    ("flags", ingest.direct_to_binary),
    ("thread", ingest.discord_thread_to_binary),
    ("sticker_items", ingest.discord_sticker_items_to_binary),
    ("position", ingest.direct_to_binary),
    ("poll", ingest.discord_poll_to_binary),
    ("call", ingest.discord_call_to_binary),
]
discord_text_canonical: types.Canonical = [
    ("content", ingest.direct_to_binary),
]
discord_mention_user_canonical: types.Canonical = [
    ("message_id", ingest.direct_to_binary),
    ("user_id", ingest.direct_to_binary),
]
discord_mention_channel_canonical: types.Canonical = [
    ("message_id", ingest.direct_to_binary),
    ("channel_id", ingest.direct_to_binary),
]

## REACTIONS

discord_reaction_canonical: types.Canonical = [
    ("message_id", ingest.direct_to_binary),
    ("user_id", ingest.direct_to_binary),
    ("emoji", ingest.discord_emoji_to_binary),
]

## MEMBERS

discord_member_canonical: types.Canonical = [
    ("guild_id", ingest.direct_to_binary),
    ("user_id", ingest.direct_to_binary),
    ("nick", ingest.direct_to_binary),
    ("avatar", ingest.direct_to_binary),
    ("roles", ingest.array_direct_to_binary),
    ("joined_at", ingest.direct_to_binary),
    ("premium_since", ingest.direct_to_binary),
    ("deaf", ingest.boolean_to_binary),
    ("mute", ingest.boolean_to_binary),
    ("pending", ingest.boolean_to_binary),
    ("flags", ingest.direct_to_binary),
    ("permissions", ingest.direct_to_binary),
    ("communication_disabled_until", ingest.direct_to_binary),
    ("avatar_decoration_data", ingest.discord_avatar_decoration_data_to_binary),
]

discord_user_canonical: types.Canonical = [
    ("id", ingest.direct_to_binary),
    ("username", ingest.direct_to_binary),
    ("global_name", ingest.direct_to_binary),
    ("avatar", ingest.direct_to_binary),
    ("bot", ingest.boolean_to_binary),
    ("system", ingest.boolean_to_binary),
    ("locale", ingest.direct_to_binary),
    ("email", ingest.direct_to_binary),
    ("flags", ingest.direct_to_binary),
]
