import argparse
from typing import cast

from delta.tables import DeltaTable
from pyspark.sql import SparkSession

from trend_etl.functions.delta import (
    ao_merge_using_ingestion_key,
)
from trend_etl.transform.reactions import from_bronze_get_reactions_df_ao

program = "load_reactions_ao"


def main() -> None:
    parser = argparse.ArgumentParser(description=f"Arguments for Trend ETL/{program}.")

    parser.add_argument(
        "--bronze-table",
        dest="bronze_table",
        required=True,
        help="The unity catalog table to use",
    )

    parser.add_argument(
        "--dst-table",
        dest="dst_table",
        required=True,
        help="The unity catalog table to use",
    )

    parser.add_argument(
        "--checkpoints-path",
        dest="checkpoints_path",
        required=True,
        help="The path to the folder (in S3) to store the checkpoints for this table",
    )

    args = parser.parse_args()

    spark = cast(SparkSession, SparkSession.builder.appName(program).getOrCreate())

    streaming = spark.readStream.format("delta").table(args.bronze_table)

    prepared = from_bronze_get_reactions_df_ao(streaming)

    delta_table = DeltaTable.forName(spark, args.dst_table)

    (
        prepared.writeStream.format("delta")
        .foreachBatch(
            lambda batch_df, batch_id: ao_merge_using_ingestion_key(
                delta_table, batch_df
            )
        )
        .option("checkpointLocation", args.checkpoints_path)
        .outputMode("append")
        .trigger(availableNow=True)
        .start()
    )


if __name__ == "__main__":
    main()
