import argparse
from typing import cast

from pyspark.sql import SparkSession

import trend_etl.transform.gold as gold

program = "materialize_nlp_results"


def main() -> None:
    parser = argparse.ArgumentParser(description=f"Arguments for Trend ETL/{program}.")

    # SRC table
    parser.add_argument(
        "--src-table",
        dest="src_table",
        required=True,
        help="The unity catalog table to use",
    )

    # DST table
    parser.add_argument(
        "--dst-table",
        dest="dst_table",
        required=True,
        help="The unity catalog table to use",
    )

    # Facebook
    parser.add_argument(
        "--groups-table",
        dest="groups_table",
        required=True,
        help="The unity catalog table to use",
    )
    parser.add_argument(
        "--posts-table",
        dest="posts_table",
        required=True,
        help="The unity catalog table to use",
    )
    parser.add_argument(
        "--comments-table",
        dest="comments_table",
        required=True,
        help="The unity catalog table to use",
    )

    # Reddit
    parser.add_argument(
        "--subreddits-table",
        dest="subreddits_table",
        required=True,
        help="The unity catalog table to use",
    )
    parser.add_argument(
        "--submissions-table",
        dest="submissions_table",
        required=True,
        help="The unity catalog table to use",
    )
    parser.add_argument(
        "--replies-table",
        dest="replies_table",
        required=True,
        help="The unity catalog table to use",
    )

    # Discord
    parser.add_argument(
        "--guilds-table",
        dest="guilds_table",
        required=True,
        help="The unity catalog table to use",
    )
    parser.add_argument(
        "--channels-table",
        dest="channels_table",
        required=True,
        help="The unity catalog table to use",
    )
    parser.add_argument(
        "--messages-table",
        dest="messages_table",
        required=True,
        help="The unity catalog table to use",
    )

    args = parser.parse_args()

    spark = cast(SparkSession, SparkSession.builder.appName(program).getOrCreate())

    groups = spark.read.format("delta").table(args.groups_table)
    posts = spark.read.format("delta").table(args.posts_table)
    comments = spark.read.format("delta").table(args.comments_table)
    facebook_set = gold.facebook_subset(groups, posts, comments)

    subreddits = spark.read.format("delta").table(args.subreddits_table)
    submissions = spark.read.format("delta").table(args.submissions_table)
    replies = spark.read.format("delta").table(args.replies_table)
    reddit_set = gold.reddit_subset(subreddits, submissions, replies)

    guilds = spark.read.format("delta").table(args.guilds_table)
    channels = spark.read.format("delta").table(args.channels_table)
    messages = spark.read.format("delta").table(args.messages_table)
    discord_set = gold.discord_subset(guilds, channels, messages)

    src_table = spark.read.format("delta").table(args.src_table)

    (
        facebook_set.unionAll(reddit_set)
        .unionAll(discord_set)
        .join(src_table, "text_version")
        .write.format("delta")
        .mode("overwrite")
        .saveAsTable(args.dst_table)
    )


if __name__ == "__main__":
    main()
