from pyspark.sql import DataFrame
import pyspark.sql.functions as fn


def reddit_subset(
    subreddits: DataFrame,
    reddit_submissions: DataFrame,
    reddit_replies: DataFrame,
) -> DataFrame:
    return (
        reddit_submissions.alias("rs")
        .join(
            subreddits.alias("s"),
            fn.col("s.source_keys.name") == fn.col("rs.source_keys.subreddit_id"),
            "left_outer",
        )
        .select(
            fn.col("s.group_id").alias("group_id"),
            fn.col("rs.doc_uuid").alias("doc_uuid"),
            fn.col("rs.text_version").alias("text_version"),
        )
        .unionAll(
            reddit_replies.alias("rr")
            .join(
                subreddits.alias("s"),
                fn.col("s.source_keys.name") == fn.col("rr.source_keys.subreddit_id"),
                "left_outer",
            )
            .select(
                fn.col("s.group_id").alias("group_id"),
                fn.col("rr.doc_uuid").alias("doc_uuid"),
                fn.col("rr.text_version").alias("text_version"),
            )
        )
    )


def facebook_subset(
    facebook_groups: DataFrame,
    facebook_posts: DataFrame,
    facebook_comments: DataFrame,
) -> DataFrame:
    return (
        facebook_posts.alias("p")
        .join(
            facebook_groups.alias("g"),
            fn.col("g.source_keys.id") == fn.col("p.source_keys.group_id"),
            "left_outer",
        )
        .select(
            fn.col("g.group_id").alias("group_id"),
            fn.col("p.doc_uuid").alias("doc_uuid"),
            fn.col("p.text_version").alias("text_version"),
        )
        .unionAll(
            facebook_comments.alias("c")
            .join(
                facebook_groups.alias("g"),
                fn.col("g.source_keys.id") == fn.col("c.source_keys.group_id"),
                "left_outer",
            )
            .select(
                fn.col("g.group_id").alias("group_id"),
                fn.col("c.doc_uuid").alias("doc_uuid"),
                fn.col("c.text_version").alias("text_version"),
            )
        )
    )


def discord_subset(
    discord_guilds: DataFrame,
    discord_channels: DataFrame,
    discord_messages: DataFrame,
) -> DataFrame:
    return (
        discord_messages.alias("m")
        .join(
            discord_channels.alias("c"),
            fn.col("c.source_keys.id") == fn.col("m.source_keys.channel_id"),
            "left_outer",
        )
        .join(
            discord_guilds.alias("g"),
            fn.col("g.source_keys.id") == fn.col("c.source_keys.guild_id"),
            "left_outer",
        )
        .select(
            fn.col("g.group_id").alias("group_id"),
            fn.col("m.doc_uuid").alias("doc_uuid"),
            fn.col("m.text_version").alias("text_version"),
        )
    )
